"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Codemirror = void 0;
const react_1 = __importStar(require("react"));
const classnames_1 = __importDefault(require("classnames"));
const codemirror_1 = __importDefault(require("codemirror"));
const fast_deep_equal_1 = __importDefault(require("fast-deep-equal"));
const Codemirror = (props) => {
    const { autoFocus, className, codeMirrorInstance = codemirror_1.default, name, options, onChange, onCursorActivity, onFocusChange, onScroll, preserveScrollPosition = false, value = '', } = props;
    const codemirrorInstance = (0, react_1.useRef)(null);
    const [isFocused, setFocused] = (0, react_1.useState)(false);
    const textareaNode = (0, react_1.useRef)(null);
    const editorClassName = (0, classnames_1.default)('ReactCodeMirror', isFocused ? 'ReactCodeMirror--focused' : null, className);
    function setOptionIfChanged(optionName, newValue) {
        const codeMirror = codemirrorInstance.current;
        if (!codeMirror) {
            return;
        }
        const oldValue = codeMirror.getOption(optionName);
        if (!(0, fast_deep_equal_1.default)(oldValue, newValue)) {
            codeMirror.setOption(optionName, newValue);
        }
    }
    function focusChanged(focused, instance, event) {
        setFocused(focused);
        if (onFocusChange) {
            onFocusChange(focused, instance, event);
        }
    }
    function cursorActivity(cm) {
        if (onCursorActivity) {
            onCursorActivity(cm);
        }
    }
    function scrollChanged(cm) {
        if (onScroll) {
            onScroll(cm.getScrollInfo());
        }
    }
    function codemirrorValueChanged(instance, changeObj) {
        if (onChange && changeObj.origin !== 'setValue') {
            onChange(instance.getValue(), changeObj);
        }
    }
    (0, react_1.useEffect)(() => {
        if (textareaNode.current instanceof HTMLTextAreaElement) {
            codemirrorInstance.current = codeMirrorInstance.fromTextArea(textareaNode.current, options);
            const codeMirror = codemirrorInstance.current;
            if (!codeMirror) {
                return () => null;
            }
            codeMirror.on('change', codemirrorValueChanged);
            codeMirror.on('cursorActivity', cursorActivity);
            codeMirror.on('focus', focusChanged.bind(null, true));
            codeMirror.on('blur', focusChanged.bind(null, false));
            codeMirror.on('scroll', scrollChanged);
        }
        return () => {
            const codeMirror = codemirrorInstance.current;
            if (!codeMirror) {
                return;
            }
            if (codeMirror) {
                codeMirror.toTextArea();
            }
        };
    }, [textareaNode.current]);
    (0, react_1.useEffect)(() => {
        const codeMirror = codemirrorInstance.current;
        if (!codeMirror) {
            return;
        }
        codeMirror.setValue(value);
    }, [value]);
    (0, react_1.useEffect)(() => {
        const codeMirror = codemirrorInstance.current;
        if (!codeMirror) {
            return;
        }
        if (preserveScrollPosition) {
            const prevScrollPosition = codeMirror.getScrollInfo();
            codeMirror.setValue(value);
            codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
        }
        if (typeof options === 'object') {
            Object.entries(options).forEach(([optionName, optionValue]) => 
            // @ts-ignore
            setOptionIfChanged(optionName, optionValue));
        }
    }, [value, JSON.stringify(options)]);
    return (react_1.default.createElement("div", { className: editorClassName },
        react_1.default.createElement("textarea", { ref: textareaNode, name: name, defaultValue: value, autoComplete: "off", autoFocus: autoFocus })));
};
exports.Codemirror = Codemirror;
